<?php

namespace Teknisa\CADFIN\Controllers;

use Teknisa\CADFIN\Services\Bco51100TipodeDebitoBancarioService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\CADFIN\Controllers\TeknisaCrud;

class GeralpagTipoajpagarController extends TeknisaCrud {
    protected $dataSourceName = "geralpag_tipoajpagar_tipobaixapag_tpcredbanc";
    protected $connection;
    protected $newCode;

    public function __construct($crudService, $parameterBag, $entityManager, $newCode) {
		parent::__construct($crudService, $parameterBag);
		$this->newCode = $newCode;
		$this->connection = $entityManager->getConnection();
	}

    public function save(DTO\Request $request, DTO\Response $response){
    	$dataSet = $request->getDataSet();
    	$rows = $dataSet->getRows();
    	//Row[0] =
    	//
    	//
    	//Cada row coresponde a uma tabela diferente que deve ser salva
    	$rowGeralRec = $rows[0];

		$method = $request->getMethod();
        $routePath = $request->getRoutePath();
        $userId = $request->getUserId();

    	$this->connection->beginTransaction();

    	try{
            $this->saveRow($rowGeralRec, 'geral_pag', $method, $routePath, $userId, $response);

            $paramsParamGeral = array(
               "IDUTLPROCPAG" => $rowGeralRec["IDUTLPROCPAG"],
               "CDGROPERPROP" => $rowGeralRec["CDGROPERPROP"],
               "CDPARGERAL" => "0001"
            );

            $this->atualizaTabelaParamGeral($paramsParamGeral);

            $this->saveOperadores($rowGeralRec);


			$this->connection->commit();
		} catch (\Exception $e){
			$response->setError(new Error($e->getMessage(), 500));
			$this->connection->rollBack();
		}
    }

    public function saveRow($row, $dataSourceName, $method, $routePath, $userId, $response){
    	$request = $this->buildResquest($row, $dataSourceName, $method, $routePath, $userId);
    	$this->dataSourceName = $dataSourceName;
    	parent::save($request, $response);
    	$this->dataSourceName = 'geralrec_tipoajreceber';
    }

    public function buildResquest($row, $dataSourceName, $method, $routePath, $userId){
        $dataSet = new DataSet($dataSourceName, array($row));
        $request = new DTO\Request\DataSet($dataSet, $method, $routePath, $userId);
        return $request;
    }

    public function saveTabedsOperadores(DTO\Request $request, DTO\Response $response) {
     try{

        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();

        $this->service->saveTabedsOperadores($rows);
        $response->addDataSet(new DataSet('retorno', array('success' => 'S')));

     } catch (\Exception $e){
       $response->setError(new Error($e->getMessage(), 500));
     }
   }


    public function atualizaTabelaParamGeral($params){

        $sql = "UPDATE PARAMGERAL SET IDUTLPROCPAG =:IDUTLPROCPAG,
                                      CDGROPERPROP =:CDGROPERPROP
                WHERE CDPARGERAL =:CDPARGERAL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);

    }

    public function saveOperadores($row){

            if(empty($row["CDGRUPOPERPRVC"])) {
                $idautgrupop = 'PRVC';
                $verificaOperadoresInseridos = $this->verificaOperadoresInseridos($idautgrupop);

                foreach($verificaOperadoresInseridos as $dados){
                    $param =  array(
                        "CDGRUPOPER" => $dados["CDGRUPOPER"]
                    );
                    $this->removeGrupoOperadores($param);
                }
            } else {

                // Adiciona ou remove (Grupo de Operadores) referente a Controla Prorrogacao de Vencimento
                $idautgrupop = 'PRVC';
                $verificaOperadoresInseridos = $this->verificaOperadoresInseridos($idautgrupop);
                $operadoresExistentes = array();

                if(!is_array($row["CDGRUPOPERPRVC"])){
                    $row["CDGRUPOPERPRVC"] = array($row["CDGRUPOPERPRVC"]);
                }

                foreach($verificaOperadoresInseridos as $dados){
                    array_push($operadoresExistentes, $dados["CDGRUPOPER"]);
                    if(!in_array($dados["CDGRUPOPER"],$row["CDGRUPOPERPRVC"])){
                        $param =  array(
                            "CDGRUPOPER" => $dados["CDGRUPOPER"]
                        );
                        $this->removeGrupoOperadores($param);
                    }
                }

                $nrautgrupop = $this->newCode->getCodeByTableName('AUTGRUPOP',$row["NRORG"]);

                foreach ($row["CDGRUPOPERPRVC"] as $indiceContPross) {
                    if(!in_array($indiceContPross,$operadoresExistentes)){
                        if(empty($verificaOperadoresInseridos)){
                            $dados = array(
                              "CDGRUPOPER" => $indiceContPross,
                              "IDAUTGRUPOP" => "PRVC",
                              "NRAUTGRUPOP" => substr($nrautgrupop, -3)
                            );
                            $this->insereAutoGroupPop($dados);

                        } else {
                            $dados = array(
                              "CDGRUPOPER" => $indiceContPross,
                              "IDAUTGRUPOP" => "PRVC",
                              "NRAUTGRUPOP" => $verificaOperadoresInseridos[0]["NRAUTGRUPOP"]
                            );
                            $this->insereAutoGroupPop($dados);
                        }

                    }
                }
            }

            if(empty($row["CDGRUPOPEREXBP"])){

                $idautgrupop = 'EXBP';
                $verificaOperadoresExclusaoBaixa = $this->verificaOperadoresInseridos($idautgrupop);

                foreach($verificaOperadoresExclusaoBaixa as $dados){
                    $param =  array(
                        "CDGRUPOPER" => $dados["CDGRUPOPER"]
                    );
                    $this->removeGrupoOperadores($param);
                }

            } else {
                // Adiciona ou remove (Grupo de Operadores) referente a Controla Exclusao de Baixa (Recebimento)
                $idautgrupop = 'EXBP';
                $verificaOperadoresExclusaoBaixa = $this->verificaOperadoresInseridos($idautgrupop);
                $operadoresExclusaoBaixa = array();

                foreach($verificaOperadoresExclusaoBaixa as $dados){
                    array_push($operadoresExclusaoBaixa, $dados["CDGRUPOPER"]);
                    if(!in_array($dados["CDGRUPOPER"],$row["CDGRUPOPEREXBP"])){
                        $param =  array(
                            "CDGRUPOPER" => $dados["CDGRUPOPER"]
                        );
                        $this->removeGrupoOperadores($param);
                    }
                }

                $nrautgrupop = $this->newCode->getCodeByTableName('AUTGRUPOP',$row["NRORG"]);
                foreach ($row["CDGRUPOPEREXBP"] as $indiceExcBaixa) {
                    if(!in_array($indiceExcBaixa,$operadoresExclusaoBaixa)){
                        if(empty($verificaOperadoresExclusaoBaixa)){
                            $dados = array(
                              "CDGRUPOPER" => $indiceExcBaixa,
                              "IDAUTGRUPOP" => "EXBP",
                              "NRAUTGRUPOP" =>  substr($nrautgrupop, -3)
                            );
                            $this->insereAutoGroupPop($dados);

                        } else {
                            $dados = array(
                              "CDGRUPOPER" => $indiceExcBaixa,
                              "IDAUTGRUPOP" => "EXBP",
                              "NRAUTGRUPOP" => $verificaOperadoresExclusaoBaixa[0]["NRAUTGRUPOP"]
                            );
                            $this->insereAutoGroupPop($dados);
                        }

                    }
                }
            }

            if(empty($row["CDGRUPOPERCRMP"])){

                $idautgrupop = 'CRMP';
                $verificaOperadoresRemessaBoleto = $this->verificaOperadoresInseridos($idautgrupop);

                foreach($verificaOperadoresRemessaBoleto as $dados){
                    $param =  array(
                        "CDGRUPOPER" => $dados["CDGRUPOPER"]
                    );
                    $this->removeGrupoOperadores($param);
                }

            } else {

                // Adiciona ou remove (Grupo de Operadores) referente a Controla Cancelamento de Remessa de Boleto
                $idautgrupop = 'CRMP';
                $verificaOperadoresRemessaBoleto = $this->verificaOperadoresInseridos($idautgrupop);
                $operadoresRemessaBoleto = array();

                foreach($verificaOperadoresRemessaBoleto as $dados){
                    array_push($operadoresRemessaBoleto, $dados["CDGRUPOPER"]);
                    if(!in_array($dados["CDGRUPOPER"],$row["CDGRUPOPERCRMP"])){
                        $param =  array(
                            "CDGRUPOPER" => $dados["CDGRUPOPER"]
                        );
                        $this->removeGrupoOperadores($param);
                    }
                }

                $nrautgrupop = $this->newCode->getCodeByTableName('AUTGRUPOP',$row["NRORG"]);
                foreach ($row["CDGRUPOPERCRMP"] as $indiceRemBoleto) {
                    if(!in_array($indiceRemBoleto,$operadoresRemessaBoleto)){
                        if(empty($verificaOperadoresRemessaBoleto)){
                            $dados = array(
                              "CDGRUPOPER" => $indiceRemBoleto,
                              "IDAUTGRUPOP" => "CRMP",
                              "NRAUTGRUPOP" => substr($nrautgrupop, -3)
                            );
                            $this->insereAutoGroupPop($dados);
                        } else{
                            $dados = array(
                              "CDGRUPOPER" => $indiceRemBoleto,
                              "IDAUTGRUPOP" => "CRMP",
                              "NRAUTGRUPOP" => $verificaOperadoresRemessaBoleto[0]["NRAUTGRUPOP"]
                            );
                            $this->insereAutoGroupPop($dados);
                        }
                    }
                }
            }
    }

    public function verificaOperadoresInseridos($idautgrupop){

        $sql = "SELECT A.CDGRUPOPER, A.NRAUTGRUPOP
                  FROM AUTGRUPOP A
                 WHERE  A.IDAUTGRUPOP =  '$idautgrupop'";
        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }

    public function insereAutoGroupPop($dados){
        $sql = "INSERT INTO AUTGRUPOP (CDGRUPOPER, IDAUTGRUPOP, NRAUTGRUPOP)
                            VALUES(:CDGRUPOPER, :IDAUTGRUPOP,:NRAUTGRUPOP)";
        $std = $this->connection->prepare($sql);
        $std->execute($dados);
    }

    public function removeGrupoOperadores($param){

        $sql = "DELETE FROM AUTGRUPOP
                WHERE  CDGRUPOPER =: CDGRUPOPER";
        $std = $this->connection->prepare($sql);
        $std->execute($param);
    }

}