<?php

namespace Teknisa\CADFIN\Modules\ContaReceber\Controllers;

use Teknisa\CADFIN\Modules\ContaReceber\Services\TipoRecebimentoService;
use Teknisa\Libs\Service\CRUD as CrudService;
use Teknisa\Libs\Util\Environment;
use Teknisa\CADFIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request\DataSet as Data;
use Zeedhi\Framework\DTO;

class TipoRecebimentoController extends TeknisaCrud{
    protected $dataSourceName = 'tprecboleto_tiporece_cartao';
    protected $parameterBagColumns = array(
        "NRORG",
        "CDFILIAL",
        "NRLANBOLREC"
    );

    /**
     * @param CrudService $crudService
     * @param ParameterBag $parameterBag
     * @param Environment $environment
     * @param TipoRecebimentoService $tiporecebimentoservice
     */
    public function __construct(CrudService $crudService, ParameterBag $parameterBag, Environment $environment, TipoRecebimentoService $tiporecebimentoservice)
    {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->tipoRecebimentoService = $tiporecebimentoservice;
    }

    /**
     * Finaliza Parcelamento Títulos
     * @param Data $dataset
     * @param Response $response
     */
    public function finalizaTipoRecebimento(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $datasource  = $dataset->getDataSet()->getRows();

            $nrorg       = $datasource[0]['nrorg'];
            $cdfilial    = $datasource[0]['cdfilial'];
            $nrlanbolrec = $datasource[0]['nrlanbolrec'];

            $tiporecebimento = $datasource[1];

            // Gera Parcelamento Título
            $this->tipoRecebimentoService->finalizaTipoRecebimento(
                $nrorg, $cdfilial, $nrlanbolrec, $tiporecebimento
            );

            // Registra Dados
            $response->addDataSet(new DataSet('TipoRecebimentoService', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deletaTipoRecebimento(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $datasource  = $dataset->getDataSet()->getRows();

            $nrorg       = $datasource[0]['nrorg'];
            $cdfilial    = $datasource[0]['cdfilial'];
            $nrlanbolrec = $datasource[0]['nrlanbolrec'];

            $tiporecebimento = $datasource[1];

            // Gera Parcelamento Título
            $this->tipoRecebimentoService->deletaTipoRecebimento(
                $nrorg, $cdfilial, $nrlanbolrec, $tiporecebimento
            );

            // Registra Dados
            $response->addDataSet(new DataSet('TipoRecebimentoService', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }


}