<?php

namespace Teknisa\CADFIN\Services;

use Teknisa\CADFIN\Factories\Ger08500ContaCorrenteFactory;
use Teknisa\Model\Fiscal\Entities\Contcorr;
use Teknisa\CADFIN\Services\FluxoDeCaixaService;
use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;

class Ger08500ContaCorrenteService{

    protected $factory;
    protected $environment;
    protected $fluxodecaixaservice;

    public function __construct(Ger08500ContaCorrenteFactory $factory, $environment, FluxoDeCaixaService $fluxodecaixaservice, EntityManager $entityManager) {
        $this->factory = $factory;
        $this->environment = $environment;
        $this->fluxodecaixaservice = $fluxodecaixaservice;
        $this->entityManager = $entityManager;
    }

    public function validarContaCorrente($row){
        $contcorr = $this->factory->getContcorr($row["CDREDUCONT"],$this->environment->getNrOrgTrab());
        if($contcorr != null){
            throw new \Exception("Impossível selecionar. Registro já existe.");
        }
    }

    public function implantaSaldoContaCorrente($row){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $saldoImplantado = $this->validaSaldoImplantado($row);
            $dtsdoccorr = $this->formataData($row['DTSDOCCORR']);
            if ($saldoImplantado == null){
            
                $sdoccorr = $this->factory->insereSaldoContaCorrente($dtsdoccorr, $row['CDREDUCONT'], $row['VRSDOCCORR'], $row['VRSDOIMPLA'], $this->environment->getCdoperador(), $this->environment->getNrOrgTrab());
                $this->entityManager->persist($sdoccorr);
                $this->entityManager->flush();
                $this->callProcedureEvolucao($row['CDREDUCONT'], $row['DTSDOCCORR'], 0, 0);
                $this->entityManager->flush();
                $connection->commit(); 
            } else if($saldoImplantado == '0'){
                $params = array(
                    'VRSDOIMPLA' => $row['VRSDOIMPLA'],
                    'CDREDUCONT' => $row['CDREDUCONT'],
                    'DTSDOCCORR' => $row['DTSDOCCORR']
                        
                );
                
                $sdoccorr = $this->factory->atualizaSaldoImplantado($params);
                $this->callProcedureEvolucao($row['CDREDUCONT'], $row['DTSDOCCORR'], 0, 0);
                $this->entityManager->flush();
                $connection->commit();
            }
            
        }catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public function validaSaldoImplantado($row){
        $dtsdoccorr = $this->formataData($row['DTSDOCCORR']);
        $existeSaldo = $this->factory->getSaldoContaCorrente($row['CDREDUCONT'], $dtsdoccorr, $this->environment->getNrOrgTrab());
        
        if ($existeSaldo){
            if($existeSaldo->getVrsdoimpla() > 0){
                throw new \Exception("Data da implantação do saldo de conta corrente já cadastrada.");
               return '1';
            } else if($existeSaldo->getVrsdoimpla() == null){
                 return '0';
            }
        }
    }

    public function excluiImplantacaoSaldo($row){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $dtsdoccorr = $this->formataData($row['DTSDOCCORR']);
            $sdoccorr = $this->factory->getSaldoContaCorrente($row['CDREDUCONT'], $dtsdoccorr, $this->environment->getNrOrgTrab());
            $this->entityManager->remove($sdoccorr);
            $this->entityManager->flush();
            $this->callProcedureEvolucao($row['CDREDUCONT'], $dtsdoccorr->format('d/m/Y'), 0, 0);
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public function evolucaoSaldo($rows){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            foreach($rows as $row){
                $this->callProcedureEvolucao($row['CDREDUCONT'], $row['DATAINICIAL'], 0, 0);
            }
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public function callProcedureEvolucao($cdreducont, $datainicial, $vrsdoccorr, $idatualiza){
        $this->factory->execProcAtualizaSaldoBancario($cdreducont, $datainicial, $vrsdoccorr, $idatualiza);
    }

    public function formataData($data){
        if(strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
    
    
    public function finalizarAlterarNossoNumero($row) {
    	$connection = $this->entityManager->getConnection();
    	$connection->beginTransaction();
    	try {
	
    		// Busca Nrorg Logado
    		$nrorg = $this->environment->getNrOrgTrab();
    		
    		/******** PRIMEIRO CAMPO ********/
    		
    		// Busca Campos da Tela
    		$cdcontador      = 'NOSSONUMERO'.$row['CDREDUCONT'];
    		$nrsequencial    = $row['BOLETOCOMPLETO'];
    		$nrsequencial = str_pad($row['BOLETOCOMPLETO'], 20, "0", STR_PAD_LEFT);
    		
    		// Busca Novo Código
    		$novocodigo = $this->factory->buscaNovocodigo($nrorg, $cdcontador);
    		
    		if($novocodigo){    	    	
    			// Atualiza Novo Código
    			$novocodigo = $this->factory->atualizaNovocodigo($novocodigo, $nrsequencial);
    		}else{
    			// Insere Novo Código
    			$novocodigo = $this->factory->insereNovocodigo($nrorg, $cdcontador, $nrsequencial);
    			
    		}
    		
    		$this->entityManager->persist($novocodigo);
    		
    		/******** SEGUNDO CAMPO ********/
    		
    		// Busca Campos da Tela
    		$cdcontadorimp   = 'NOSSONUMPREIMP'.$row['CDREDUCONT'];
    		$nrsequencialimp = $row['BOLETOPREIMP'];
    		$nrsequencialimp = str_pad($row['BOLETOPREIMP'], 20, "0", STR_PAD_LEFT);
    	
    		
    		// Busca Novo Código
    		$novocodigoimp = $this->factory->buscaNovocodigo($nrorg, $cdcontadorimp);
    				
    		if($novocodigoimp){    	    	
    			// Atualiza Novo Código
    			$novocodigoimp = $this->factory->atualizaNovocodigo($novocodigoimp, $nrsequencialimp);
    		}else{
    			// Insere Novo Código
    			$novocodigoimp = $this->factory->insereNovocodigo($nrorg, $cdcontadorimp, $nrsequencialimp);
    		}
    		
    		$this->entityManager->persist($novocodigoimp);
    		
    		$this->entityManager->flush();
    		$connection->commit();
    	} catch (\Exception $e) {
    		$connection->rollBack();
    		throw $e;
	}		
}
    

}